/*
   SPDX-FileCopyrightText: 2016 Martin Sandsmark <martin.sandsmark@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#include "kpLogCategories.h"

Q_LOGGING_CATEGORY(kpLogMisc, "kp.misc")
Q_LOGGING_CATEGORY(kpLogDialogs, "kp.dialogs")
Q_LOGGING_CATEGORY(kpLogCommands, "kp.commands")
Q_LOGGING_CATEGORY(kpLogDocument, "kp.document")
Q_LOGGING_CATEGORY(kpLogTools, "kp.tools")
Q_LOGGING_CATEGORY(kpLogViews, "kp.views")
Q_LOGGING_CATEGORY(kpLogEnvironments, "kp.environments")
Q_LOGGING_CATEGORY(kpLogPixmapfx, "kp.pixmapfx")
Q_LOGGING_CATEGORY(kpLogWidgets, "kp.widgets")
Q_LOGGING_CATEGORY(kpLogMainWindow, "kp.mainwindow")
Q_LOGGING_CATEGORY(kpLogLayers, "kp.layers")
Q_LOGGING_CATEGORY(kpLogImagelib, "kp.imagelib")
