/*
   SPDX-FileCopyrightText: 2016 Martin Sandsmark <martin.sandsmark@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#ifndef KPLOGCATEGORIES_H
#define KPLOGCATEGORIES_H

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(kpLogMisc)
Q_DECLARE_LOGGING_CATEGORY(kpLogDialogs)
Q_DECLARE_LOGGING_CATEGORY(kpLogCommands)
Q_DECLARE_LOGGING_CATEGORY(kpLogDocument)
Q_DECLARE_LOGGING_CATEGORY(kpLogTools)
Q_DECLARE_LOGGING_CATEGORY(kpLogViews)
Q_DECLARE_LOGGING_CATEGORY(kpLogEnvironments)
Q_DECLARE_LOGGING_CATEGORY(kpLogPixmapfx)
Q_DECLARE_LOGGING_CATEGORY(kpLogWidgets)
Q_DECLARE_LOGGING_CATEGORY(kpLogMainWindow)
Q_DECLARE_LOGGING_CATEGORY(kpLogLayers)
Q_DECLARE_LOGGING_CATEGORY(kpLogImagelib)

#endif
