
/*
   SPDX-FileCopyrightText: 2003-2007 Clarence Dang <dang@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#include "environments/dialogs/imagelib/transforms/kpTransformDialogEnvironment.h"

#include "mainWindow/kpMainWindow.h"

//---------------------------------------------------------------------

kpTransformDialogEnvironment::kpTransformDialogEnvironment(kpMainWindow *mainWindow)
    : kpEnvironmentBase(mainWindow)
{
}

//---------------------------------------------------------------------

#include "moc_kpTransformDialogEnvironment.cpp"
