
/*
   SPDX-FileCopyrightText: 2003-2007 Clarence Dang <dang@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#ifndef KP_CURSOR_LIGHT_CROSS_H
#define KP_CURSOR_LIGHT_CROSS_H

class QCursor;

const QCursor *kpCursorLightCrossCreate();

#endif // KP_CURSOR_LIGHT_CROSS_H
