
/*
   SPDX-FileCopyrightText: 2003-2007 Clarence Dang <dang@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#include "kpSubWindow.h"

kpSubWindow::kpSubWindow(QWidget *parent)
    : QDialog(parent)
{
}
