
/*
   SPDX-FileCopyrightText: 2003-2007 Clarence Dang <dang@kde.org>

   SPDX-License-Identifier: BSD-2-Clause
*/

#ifndef KP_IMAGE_H
#define KP_IMAGE_H

#include <QImage>

typedef QImage kpImage;

#endif // KP_IMAGE_H
